@@ -1,4 +1,4 @@
-/*
+	/*
  *  Copyright (C) 2003 Marco Pesenti Gritti
  *  Copyright (C) 2003 Christian Persch
  *
@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-cookie-manager.h,v 1.5 2004/02/20 11:49:42 chpe Exp $
+ *  $Id: ephy-cookie-manager.h,v 1.8 2005/01/14 23:54:52 chpe Exp $
  */
 
 #ifndef EPHY_COOKIE_MANAGER_H
@@ -36,27 +36,8 @@ G_BEGIN_DECLS
 
 #define EPHY_TYPE_COOKIE			(ephy_cookie_get_type ())
 
-typedef struct EphyCookieManager	EphyCookieManager;
-typedef struct EphyCookieManagerIface	EphyCookieManagerIface;
-
-typedef enum
-{
-	EPHY_COOKIE_P3P_STATE_UNKNOWN,
-	EPHY_COOKIE_P3P_STATE_ACCEPTED,
-	EPHY_COOKIE_P3P_STATE_DOWNGRADED,
-	EPHY_COOKIE_P3P_STATE_FLAGGED,
-	EPHY_COOKIE_P3P_STATE_REJECTED
-} EphyCookieP3PState;
-
-typedef enum
-{
-	EPHY_COOKIE_P3P_POLICY_UNKNOWN,
-	EPHY_COOKIE_P3P_POLICY_NONE,
-	EPHY_COOKIE_P3P_POLICY_NO_CONSENT,
-	EPHY_COOKIE_P3P_POLICY_IMPLICIT_CONSENT,
-	EPHY_COOKIE_P3P_POLICY_EXPLICIT_CONSENT,
-	EPHY_COOKIE_P3P_POLICY_NO_II,
-} EphyCookieP3PPolicy;
+typedef struct _EphyCookieManager	EphyCookieManager;
+typedef struct _EphyCookieManagerIface	EphyCookieManagerIface;
 
 typedef struct
 {
@@ -66,13 +47,11 @@ typedef struct
 	char *path;
 	gulong expires;
 	glong real_expires;
-	guint is_secure : 1;
-	guint is_session : 1;
-	guint p3p_state : 3;
-	guint p3p_policy : 3;
+        guint is_secure : 1;
+        guint is_session : 1;
 } EphyCookie;
 
-struct EphyCookieManagerIface
+struct _EphyCookieManagerIface
 {
 	GTypeInterface base_iface;
 
