@@ -16,12 +16,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-embed-persist.c,v 1.13 2004/03/27 21:51:59 adamh Exp $
+ *  $Id: ephy-embed-persist.c,v 1.23 2005/02/02 19:27:07 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-embed-persist.h"
 #include "mozilla-embed-persist.h"
@@ -43,7 +41,7 @@ enum
 
 #define EPHY_EMBED_PERSIST_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_EMBED_PERSIST, EphyEmbedPersistPrivate))
 
-struct EphyEmbedPersistPrivate
+struct _EphyEmbedPersistPrivate
 {
 	char *dest;
 	char *source;
@@ -51,7 +49,7 @@ struct EphyEmbedPersistPrivate
 	char *persist_key;
 	EphyEmbed *embed;
 	long max_size;
-	EmbedPersistFlags flags;
+	EphyEmbedPersistFlags flags;
 	GtkWindow *fc_parent;
 };
 
@@ -65,7 +63,7 @@ ephy_embed_persist_get_type (void)
 {
        static GType type = 0;
 
-	if (type == 0)
+	if (G_UNLIKELY (type == 0))
 	{
 		static const GTypeInfo our_info =
 		{
@@ -112,7 +110,7 @@ ephy_embed_persist_set_dest (EphyEmbedPersist *persist,
  * Sets the #EphyEmbed from which @persist will download data.
  *
  * An #EphyEmbed is absolutely required to download if @persist's
- * #EmbedPersistFlags include %EMBED_PERSIST_COPY_PAGE. Regardless, an
+ * #EphyEmbedPersistFlags include %EPHY_EMBED_PERSIST_COPY_PAGE. Regardless, an
  * #EphyEmbed should be set for <emphasis>every</emphasis> #EphyEmbedPersist,
  * since it determines request information such as the referring page.
  **/
@@ -131,7 +129,7 @@ ephy_embed_persist_set_embed (EphyEmbedPersist *persist,
  * @value: the title to be displayed by the filechooser
  *
  * Sets the title of the filechooser window. The filechooser will only be
- * displayed if %EMBED_PERSIST_ASK_DESTINATION has been set with
+ * displayed if %EPHY_EMBED_PERSIST_ASK_DESTINATION has been set with
  * ephy_embed_persist_set_flags().
  **/
 void
@@ -149,28 +147,36 @@ ephy_embed_persist_set_fc_title (EphyEmbedPersist *persist,
  * @value: the #EphyWindow which should be the filechooser's parent
  *
  * Sets the #EphyWindow which should be @persist's filechooser's parent. The
- * filechooser will only be displayed if %EMBED_PERSIST_ASK_DESTINATION has been
+ * filechooser will only be displayed if %EPHY_EMBED_PERSIST_ASK_DESTINATION has been
  * set with ephy_embed_persist_set_flags().
  **/
 void
 ephy_embed_persist_set_fc_parent (EphyEmbedPersist *persist,
 				  GtkWindow *value)
 {
+	EphyEmbedPersistPrivate *priv;
+	GtkWindow **wptr;
+
 	g_return_if_fail (EPHY_IS_EMBED_PERSIST (persist));
 
-	persist->priv->fc_parent = value;
+	priv = persist->priv;
+
+	priv->fc_parent = value;
+	wptr = &priv->fc_parent;
+	g_object_add_weak_pointer (G_OBJECT (priv->fc_parent),
+				   (gpointer *) wptr);
 }
 
 /**
  * ephy_embed_persist_set_flags:
  * @persist: an #EphyEmbedPersist
- * @value: the desired #EmbedPersistFlags
+ * @value: the desired #EphyEmbedPersistFlags
  *
  * Sets the flags to be used for @persist's download.
  **/
 void
 ephy_embed_persist_set_flags (EphyEmbedPersist *persist,
-			      EmbedPersistFlags value)
+			      EphyEmbedPersistFlags value)
 {
 	g_return_if_fail (EPHY_IS_EMBED_PERSIST (persist));
 
@@ -301,11 +307,11 @@ ephy_embed_persist_get_fc_parent (EphyEmbedPersist *persist)
  * ephy_embed_persist_get_flags:
  * @persist: an #EphyEmbedPersist
  *
- * Returns @persist's #EmbedPersistFlags.
+ * Returns @persist's #EphyEmbedPersistFlags.
  *
- * Return value: @persist's #EmbedPersistFlags
+ * Return value: @persist's #EphyEmbedPersistFlags
  **/
-EmbedPersistFlags
+EphyEmbedPersistFlags
 ephy_embed_persist_get_flags (EphyEmbedPersist *persist)
 {
 	g_return_val_if_fail (EPHY_IS_EMBED_PERSIST (persist), 0);
@@ -440,28 +446,31 @@ ephy_embed_persist_init (EphyEmbedPersist *persist)
 {
 	persist->priv = EPHY_EMBED_PERSIST_GET_PRIVATE (persist);
 
-	LOG ("EphyEmbedPersist initialising %p", persist)
+	LOG ("EphyEmbedPersist initialising %p", persist);
 
-	persist->priv->dest = NULL;
-	persist->priv->source = NULL;
-	persist->priv->fc_title = NULL;
-	persist->priv->fc_parent = NULL;
-	persist->priv->flags = 0;
 	persist->priv->max_size = -1;
-	persist->priv->persist_key = NULL;
 }
 
 static void
 ephy_embed_persist_finalize (GObject *object)
 {
 	EphyEmbedPersist *persist = EPHY_EMBED_PERSIST (object);
+	EphyEmbedPersistPrivate *priv = persist->priv;
+	GtkWindow **wptr;
 
-	g_free (persist->priv->dest);
-	g_free (persist->priv->source);
-	g_free (persist->priv->fc_title);
-	g_free (persist->priv->persist_key);
+	g_free (priv->dest);
+	g_free (priv->source);
+	g_free (priv->fc_title);
+	g_free (priv->persist_key);
 
-	LOG ("EphyEmbedPersist finalised %p", object)
+	if (priv->fc_parent != NULL)
+	{
+		wptr = &priv->fc_parent;
+		g_object_remove_weak_pointer (G_OBJECT (priv->fc_parent),
+					      (gpointer *) wptr);
+	}
+
+	LOG ("EphyEmbedPersist finalised %p", object);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -599,7 +608,7 @@ ephy_embed_persist_cancel (EphyEmbedPersist *persist)
  *
  * Begins saving the file specified in @persist.
  *
- * If @persist's #EmbedPersistFlags include %EMBED_PERSIST_ASK_DESTINATION, a
+ * If @persist's #EphyEmbedPersistFlags include %EPHY_EMBED_PERSIST_ASK_DESTINATION, a
  * filechooser dialog will be shown first.
  *
  * The file will continue to download in the background until either the
@@ -623,7 +632,7 @@ ephy_embed_persist_save (EphyEmbedPersist *persist)
  *
  * The download is synchronous. An #EphyEmbed must be specified with
  * ephy_embed_persist_set_embed(). The function implicitly assumes that
- * @persist's #EmbedPersistFlags include %EMBED_PERSIST_COPY_PAGE. If @persist's
+ * @persist's #EphyEmbedPersistFlags include %EPHY_EMBED_PERSIST_COPY_PAGE. If @persist's
  * #EphyEmbed has not finished downloading, this function will only return the
  * portion of data which has already been downloaded.
  *
@@ -638,4 +647,3 @@ ephy_embed_persist_to_string (EphyEmbedPersist *persist)
        EphyEmbedPersistClass *klass = EPHY_EMBED_PERSIST_GET_CLASS (persist);
        return klass->to_string (persist);
 }
-
