@@ -36,7 +36,7 @@
  *
  * ***** END LICENSE BLOCK *****
  *
- * $Id: EphyHeaderSniffer.cpp,v 1.41 2005/01/18 19:33:24 chpe Exp $
+ * $Id: EphyHeaderSniffer.cpp,v 1.42 2005/01/28 22:23:38 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -77,7 +77,7 @@ EphyHeaderSniffer::EphyHeaderSniffer (nsIWebBrowserPersist* aPersist, MozillaEmb
 , mDocument(aDocument)
 , mPostData(aPostData)
 {
-        LOG ("EphyHeaderSniffer ctor (%p)", this)
+        LOG ("EphyHeaderSniffer ctor (%p)", this);
 
         nsCOMPtr<nsIWindowWatcher> watcher
                 (do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
@@ -92,7 +92,7 @@ EphyHeaderSniffer::EphyHeaderSniffer (nsIWebBrowserPersist* aPersist, MozillaEmb
 
 EphyHeaderSniffer::~EphyHeaderSniffer()
 {
-	LOG ("EphyHeaderSniffer dtor (%p)", this)
+	LOG ("EphyHeaderSniffer dtor (%p)", this);
 
 	if (mSingle)
 	{
@@ -384,7 +384,7 @@ nsresult EphyHeaderSniffer::PerformSave (nsIURI* inOriginalURI)
 
 nsresult EphyHeaderSniffer::InitiateDownload (nsILocalFile *aDestFile)
 {
-	LOG ("Initiating download")
+	LOG ("Initiating download");
 
 	return InitiateMozillaDownload (mDocument, mURL, aDestFile,
 					mContentType.get(), mOriginalURI, mEmbedPersist,
