@@ -17,14 +17,12 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EventContext.cpp,v 1.39.4.3 2004/11/13 19:56:49 chpe Exp $
+ *  $Id: EventContext.cpp,v 1.49 2005/02/02 19:48:10 chpe Exp $
  */
 
 #include "mozilla-config.h"
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "EventContext.h"
 #include "EphyUtils.h"
@@ -191,7 +189,7 @@ nsresult EventContext::Unescape (const nsACString &aEscaped, nsACString &aUnesca
 	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
 	nsEmbedString unescaped;
-	rv = escaper->UnEscapeNonAsciiURI (encoding, aEscaped, unescaped);
+	rv = escaper->UnEscapeURIForUI (encoding, aEscaped, unescaped);
 	NS_ENSURE_TRUE (NS_SUCCEEDED (rv) && unescaped.Length(), NS_ERROR_FAILURE);
 
 	NS_UTF16ToCString (unescaped, NS_CSTRING_ENCODING_UTF8, aUnescaped);
@@ -214,7 +212,7 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 
 	mEmbedEvent = info;
 
-	info->context = EMBED_CONTEXT_DOCUMENT;
+	info->context = EPHY_EMBED_CONTEXT_DOCUMENT;
 
 	nsCOMPtr<nsIDOMNode> node = do_QueryInterface(EventTarget, &rv);
 	if (NS_FAILED(rv) || !node) return NS_ERROR_FAILURE;
@@ -234,7 +232,7 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 	nsCOMPtr<nsIDOMXULDocument> xul_document = do_QueryInterface(domDoc);
 	if (xul_document)
 	{
-		info->context = EMBED_CONTEXT_NONE;
+		info->context = EPHY_EMBED_CONTEXT_NONE;
 		return NS_ERROR_FAILURE;
 	}
 
@@ -259,7 +257,7 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 
 		if (g_ascii_strcasecmp (tag.get(), "img") == 0)
 		{
-			info->context |= EMBED_CONTEXT_IMAGE;
+			info->context |= EPHY_EMBED_CONTEXT_IMAGE;
 
 			nsEmbedString img;
 			nsCOMPtr <nsIDOMHTMLImageElement> image = 
@@ -276,7 +274,7 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 		}
 		else if (g_ascii_strcasecmp (tag.get(), "textarea") == 0)
 		{
-			info->context |= EMBED_CONTEXT_INPUT;
+			info->context |= EPHY_EMBED_CONTEXT_INPUT;
 		}
 		else if (g_ascii_strcasecmp (tag.get(), "object") == 0)
 		{
@@ -293,7 +291,7 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 			// MIME types are always lower case
 			if (g_str_has_prefix (cValue.get(), "image/"))
 			{
-				info->context |= EMBED_CONTEXT_IMAGE;
+				info->context |= EPHY_EMBED_CONTEXT_IMAGE;
 				
 				nsEmbedString img;
 				
@@ -308,7 +306,7 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 			}
 			else
 			{
-				info->context = EMBED_CONTEXT_NONE;
+				info->context = EPHY_EMBED_CONTEXT_NONE;
 				return NS_OK;
 			}
 		}
@@ -363,7 +361,7 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 
 			if (g_ascii_strcasecmp (cValue.get(), "simple") == 0)
 			{
-				info->context |= EMBED_CONTEXT_LINK;
+				info->context |= EPHY_EMBED_CONTEXT_LINK;
 				dom_elem->GetAttributeNS (nsEmbedString(xlinknsLiteral),
 							  nsEmbedString(hrefLiteral), value);
 				
@@ -405,20 +403,23 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 				{
 					/* cut "mailto:" */
 					href.Cut (0, 7);
-					// FIXME: cut any chars after "?"
+
+					char *str = g_strdup (href.get());
+					g_strdelimit (str, "?", '\0');
 
 					nsEmbedCString unescapedHref;
-					rv = Unescape (href, unescapedHref);
+					rv = Unescape (nsEmbedCString(str), unescapedHref);
 					if (NS_SUCCEEDED (rv) && unescapedHref.Length())
 					{
 						SetStringProperty ("email", unescapedHref.get());
-						info->context |= EMBED_CONTEXT_EMAIL_LINK;
+						info->context |= EPHY_EMBED_CONTEXT_EMAIL_LINK;
 					}
+					g_free (str);
 				}
 				
 				if (anchor && tmp.Length()) 
 				{
-					info->context |= EMBED_CONTEXT_LINK;
+					info->context |= EPHY_EMBED_CONTEXT_LINK;
 
 					SetStringProperty ("link", tmp);
 					CheckLinkScheme (tmp);
@@ -475,12 +476,12 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 			}
 			else if (g_ascii_strcasecmp (tag.get(), "option") == 0)
 			{
-				info->context = EMBED_CONTEXT_NONE;
+				info->context = EPHY_EMBED_CONTEXT_NONE;
 				return NS_OK;
 			}
-			if (g_ascii_strcasecmp (tag.get(), "area") == 0)
+			else if (g_ascii_strcasecmp (tag.get(), "area") == 0)
 			{
-				info->context |= EMBED_CONTEXT_LINK;
+				info->context |= EPHY_EMBED_CONTEXT_LINK;
 				nsCOMPtr <nsIDOMHTMLAreaElement> area =
 						do_QueryInterface(node, &rv);
 				if (NS_SUCCEEDED(rv) && area)
@@ -500,7 +501,7 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 			}
 			else if (g_ascii_strcasecmp (tag.get(), "textarea") == 0)
 			{
-				info->context |= EMBED_CONTEXT_INPUT;
+				info->context |= EPHY_EMBED_CONTEXT_INPUT;
 			}
 
 			if (!has_background)
@@ -516,8 +517,6 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
                                                 return NS_ERROR_FAILURE;
 					SetStringProperty ("background_image",
 						           bgimg.get());
-					if (NS_FAILED (rv))
-						return NS_ERROR_FAILURE;
 
 					has_background = PR_TRUE;
 				}
@@ -576,38 +575,81 @@ nsresult EventContext::GetCSSBackground (nsIDOMNode *node, nsAString& url)
 	return NS_OK;
 }
 
+nsresult EventContext::GetTargetCoords (nsIDOMEventTarget *aTarget, PRInt32 *aX, PRInt32 *aY)
+{
+	/* Calculate the node coordinates relative to the widget origin */
+	nsCOMPtr<nsIDOMNSHTMLElement> elem (do_QueryInterface(aTarget));
+
+	PRInt32 x = 0, y = 0;
+	while (elem)
+	{
+		PRInt32 val;
+		elem->GetOffsetTop(&val);	y += val;
+		elem->GetScrollTop(&val);	y -= val;
+		elem->GetOffsetLeft(&val);	x += val;
+		elem->GetScrollLeft(&val);	x -= val;
+
+		nsCOMPtr<nsIDOMElement> parent;
+		elem->GetOffsetParent (getter_AddRefs (parent));
+		elem = do_QueryInterface(parent);
+	}
+
+	*aX = x;
+	*aY = y;
+
+	return NS_OK;
+}
+
 nsresult EventContext::GetMouseEventInfo (nsIDOMMouseEvent *aMouseEvent, MozillaEmbedEvent *info)
 {
 	/* FIXME: casting 32-bit guint* to PRUint16* below will break on big-endian */
-	PRUint16 btn = (PRUint16) -1;
+	PRUint16 btn = 1729;
 	aMouseEvent->GetButton (&btn);
 
 	switch (btn)
 	{
+		/* mozilla's button counting is one-off from gtk+'s */
 		case 0:
-			info->type = EPHY_EMBED_EVENT_MOUSE_BUTTON1;
-		break;
+			info->button = 1;
+			break;
 		case 1:
-			info->type = EPHY_EMBED_EVENT_MOUSE_BUTTON2;
-		break;
+			info->button = 2;
+			break;
 		case 2:
-			info->type = EPHY_EMBED_EVENT_MOUSE_BUTTON3;
-		break;
+			info->button = 3;
+			break;
+
+#ifdef MOZ_BROKEN_CTX_MENU_EVENT
+		case 1729:
+			/* This only appears to happen when getting a mouse context menu
+			 * signal, so map it to button 3 (right mouse button)
+			 * http://bugzilla.mozilla.org/show_bug.cgi?id=258193 */
+			info->button = 3;
+			break;
+#endif
 
 		case (PRUint16) -1:
 			/* when the user submits a form with Return, mozilla synthesises
 			 * a _mouse_ click event with btn=65535 (-1).
 			 */
-			info->type = EPHY_EMBED_EVENT_KEY;
-			break;
-
 		default:
-			g_warning ("Unknown mouse button");
+			info->button = 0;
+			break;
 	}
 
-	/* OTOH, casting only between (un)signedness is safe */
-	aMouseEvent->GetScreenX ((PRInt32*)&info->x);
-	aMouseEvent->GetScreenY ((PRInt32*)&info->y);
+	if (info->button != 0)
+	{
+		/* OTOH, casting only between (un)signedness is safe */
+		aMouseEvent->GetScreenX ((PRInt32*)&info->x);
+		aMouseEvent->GetScreenY ((PRInt32*)&info->y);
+	}
+	else /* this is really a keyboard event */
+	{
+		nsCOMPtr<nsIDOMEventTarget> eventTarget;
+		aMouseEvent->GetTarget (getter_AddRefs (eventTarget));
+
+		GetTargetCoords (eventTarget, (PRInt32*)&info->x, (PRInt32*)&info->y);
+	}
 
 	/* be sure we are not clicking on the scroolbars */
 
@@ -653,8 +695,9 @@ nsresult EventContext::GetMouseEventInfo (nsIDOMMouseEvent *aMouseEvent, Mozilla
 	aMouseEvent->GetShiftKey(&mod_key);
 	if (mod_key) info->modifier |= GDK_SHIFT_MASK;
 
-	aMouseEvent->GetMetaKey(&mod_key);
-	if (mod_key) info->modifier |= GDK_MOD2_MASK;
+	/* no need to check GetMetaKey, it's always PR_FALSE,
+	 * see widget/src/gtk2/nsWindow.cpp:InitMouseEvent
+	 */
 	
 	aMouseEvent->GetCtrlKey(&mod_key);
 	if (mod_key) info->modifier |= GDK_CONTROL_MASK;
@@ -664,10 +707,9 @@ nsresult EventContext::GetMouseEventInfo (nsIDOMMouseEvent *aMouseEvent, Mozilla
 
 nsresult EventContext::GetKeyEventInfo (nsIDOMKeyEvent *aKeyEvent, MozillaEmbedEvent *info)
 {
+	info->button = 0;
+
 	nsresult rv;
-
-	info->type = EPHY_EMBED_EVENT_KEY;
-
 	PRUint32 keyCode;
 	rv = aKeyEvent->GetKeyCode(&keyCode);
 	if (NS_FAILED(rv)) return rv;
@@ -677,25 +719,7 @@ nsresult EventContext::GetKeyEventInfo (nsIDOMKeyEvent *aKeyEvent, MozillaEmbedE
 	rv = aKeyEvent->GetTarget(getter_AddRefs(target));
 	if (NS_FAILED(rv) || !target) return NS_ERROR_FAILURE;
 
-	/* Calculate the node coordinates relative to the widget origin */
-	nsCOMPtr<nsIDOMNSHTMLElement> elem = do_QueryInterface(target, &rv);
-	if (NS_FAILED(rv)) return rv;
-
-	PRInt32 x = 0, y = 0;
-	while (elem)
-	{
-		PRInt32 val;
-		elem->GetOffsetTop(&val);	y += val;
-		elem->GetScrollTop(&val);	y -= val;
-		elem->GetOffsetLeft(&val);	x += val;
-		elem->GetScrollLeft(&val);	x -= val;
-
-		nsCOMPtr<nsIDOMElement> parent;
-		elem->GetOffsetParent(getter_AddRefs(parent));
-		elem = do_QueryInterface(parent, &rv);
-	}
-	info->x = x;
-	info->y = y;
+	GetTargetCoords (target, (PRInt32*)&info->x, (PRInt32*)&info->y);
 
 	/* Context */
 	rv = GetEventContext (target, info);
@@ -766,7 +790,7 @@ nsresult EventContext::CheckInput (nsIDOMNode *aNode)
 
 	if (g_ascii_strcasecmp (value.get(), "image") == 0)
 	{
-		mEmbedEvent->context |= EMBED_CONTEXT_IMAGE;
+		mEmbedEvent->context |= EPHY_EMBED_CONTEXT_IMAGE;
 		nsCOMPtr<nsIDOMHTMLInputElement> input;
 		input = do_QueryInterface (aNode);
 		if (!input) return NS_ERROR_FAILURE;
@@ -788,7 +812,7 @@ nsresult EventContext::CheckInput (nsIDOMNode *aNode)
 		 g_ascii_strcasecmp (value.get(), "button") != 0 &&
 		 g_ascii_strcasecmp (value.get(), "checkbox") != 0)
 	{
-		mEmbedEvent->context |= EMBED_CONTEXT_INPUT;
+		mEmbedEvent->context |= EPHY_EMBED_CONTEXT_INPUT;
 	}
 
 	return NS_OK;
@@ -805,14 +829,14 @@ nsresult EventContext::CheckLinkScheme (const nsAString &link)
 	rv = uri->GetScheme (scheme);
 	if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
 
-	if (g_ascii_strcasecmp (scheme.get(), "http") ||
-	    g_ascii_strcasecmp (scheme.get(), "https") ||
-	    g_ascii_strcasecmp (scheme.get(), "ftp") ||
-	    g_ascii_strcasecmp (scheme.get(), "file") ||
-	    g_ascii_strcasecmp (scheme.get(), "data") ||
-	    g_ascii_strcasecmp (scheme.get(), "resource") ||
-	    g_ascii_strcasecmp (scheme.get(), "about") ||
-	    g_ascii_strcasecmp (scheme.get(), "gopher"))
+	if (g_ascii_strcasecmp (scheme.get(), "http") == 0 ||
+	    g_ascii_strcasecmp (scheme.get(), "https") == 0 ||
+	    g_ascii_strcasecmp (scheme.get(), "ftp") == 0 ||
+	    g_ascii_strcasecmp (scheme.get(), "file") == 0 ||
+	    g_ascii_strcasecmp (scheme.get(), "data") == 0 ||
+	    g_ascii_strcasecmp (scheme.get(), "resource") == 0 ||
+	    g_ascii_strcasecmp (scheme.get(), "about") == 0 ||
+	    g_ascii_strcasecmp (scheme.get(), "gopher") == 0)
 	{
 		SetIntProperty ("link-has-web-scheme", TRUE);
 	}
