@@ -15,17 +15,12 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
+ *  $Id: MozillaPrivate.h,v 1.7 2004/11/19 19:13:40 chpe Exp $
  */
 
-#include <nsIPrintSettings.h>
-#include <nsIDOMWindow.h>
+#include <glib.h>
 
-#include "print-dialog.h"
-
-GtkWidget *MozillaFindEmbed	(nsIDOMWindow *aDOMWindow);
-
-GtkWidget *MozillaFindGtkParent (nsIDOMWindow *aDOMWindow);
-
-NS_METHOD MozillaCollatePrintSettings (const EmbedPrintInfo *info,
-				       nsIPrintSettings *settings,
-				       gboolean preview);
+namespace MozillaPrivate
+{
+	GList   *GetPrinterList ();
+}
