@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-history-window.c,v 1.93 2005/01/21 19:33:02 jbaayen Exp $
+ *  $Id: ephy-history-window.c,v 1.94 2005/01/28 22:24:45 chpe Exp $
  */
 
 #include "config.h"
@@ -1128,7 +1128,7 @@ provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 	icon_location = ephy_node_get_property_string
 		(node, EPHY_NODE_PAGE_PROP_ICON);
 
-	LOG ("Get favicon for %s", icon_location ? icon_location : "None")
+	LOG ("Get favicon for %s", icon_location ? icon_location : "None");
 
 	if (icon_location)
 	{
