@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-window.c,v 1.254 2005/01/09 01:29:17 chpe Exp $
+ *  $Id: ephy-window.c,v 1.269 2005/02/02 19:53:26 chpe Exp $
  */
 
 #include "config.h"
@@ -44,7 +44,7 @@
 #include "ephy-file-helpers.h"
 #include "ephy-statusbar.h"
 #include "egg-editable-toolbar.h"
-#include "toolbar.h"
+#include "ephy-toolbar.h"
 #include "ephy-bookmarksbar.h"
 #include "popup-commands.h"
 #include "ephy-encoding-menu.h"
@@ -55,6 +55,7 @@
 #include "ephy-link.h"
 #include "ephy-gui.h"
 #include "ephy-notebook.h"
+#include "ephy-fullscreen-popup.h"
 
 #include <string.h>
 #include <glib/gi18n.h>
@@ -70,10 +71,14 @@
 #include <gtk/gtkmessagedialog.h>
 
 static void ephy_window_class_init		(EphyWindowClass *klass);
+static void ephy_window_link_iface_init		(EphyLinkIface *iface);
 static void ephy_window_init			(EphyWindow *gs);
 static void ephy_window_finalize		(GObject *object);
 static void ephy_window_show			(GtkWidget *widget);
-
+static EphyTab *ephy_window_open_link		(EphyLink *link,
+						 const char *address,
+						 EphyTab *tab,
+						 EphyLinkFlags flags);
 static void ephy_window_notebook_switch_page_cb	(GtkNotebook *notebook,
 						 GtkNotebookPage *page,
 						 guint page_num,
@@ -86,6 +91,12 @@ static void ephy_window_view_bookmarksbar_cb    (GtkAction *action,
 			                         EphyWindow *window);
 static void ephy_window_view_popup_windows_cb	(GtkAction *action,
 						 EphyWindow *window);
+static void sync_tab_load_status		(EphyTab *tab,
+						 GParamSpec *pspec,
+						 EphyWindow *window);
+static void sync_tab_security			(EphyTab *tab,
+						 GParamSpec *pspec,
+						 EphyWindow *window);
 static void sync_tab_zoom			(EphyTab *tab,
 						 GParamSpec *pspec,
 						 EphyWindow *window);
@@ -308,7 +319,7 @@ static GtkActionEntry ephy_popups_entries [] = {
 	{ "OpenLink", GTK_STOCK_OPEN, N_("_Open Link"), NULL,
 	  N_("Open link in this window"),
 	  G_CALLBACK (popup_cmd_open_link) },
-	{ "OpenLinkInNewWindow", NULL, N_("Open Link in _New Window"), NULL,
+	{ "OpenLinkInNewWindow", NULL, N_("Open Link in New _Window"), NULL,
 	  N_("Open link in a new window"),
 	  G_CALLBACK (popup_cmd_link_in_new_window) },
 	{ "OpenLinkInNewTab", NULL, N_("Open Link in New _Tab"), NULL,
@@ -356,8 +367,8 @@ struct _EphyWindowPrivate
 {
 	GtkWidget *main_vbox;
 	GtkWidget *menu_dock;
-	GtkWidget *exit_fullscreen_popup;
-	Toolbar *toolbar;
+	GtkWidget *fullscreen_popup;
+	EphyToolbar *toolbar;
 	GtkWidget *bookmarksbar;
 	GtkWidget *statusbar;
 	GtkUIManager *manager;
@@ -387,11 +398,11 @@ struct _EphyWindowPrivate
 	guint browse_with_caret_notifier_id;
 	guint allow_popups_notifier_id;
 
-	gboolean closing : 1;
-	gboolean has_size : 1;
-	gboolean fullscreen_mode : 1;
-	gboolean ppv_mode : 1;
-	gboolean should_save_chrome : 1;
+	guint closing : 1;
+	guint has_size : 1;
+	guint fullscreen_mode : 1;
+	guint ppv_mode : 1;
+	guint should_save_chrome : 1;
 };
 
 enum
@@ -423,66 +434,32 @@ ephy_window_get_type (void)
                         0, /* n_preallocs */
                         (GInstanceInitFunc) ephy_window_init
                 };
+		static const GInterfaceInfo link_info = 
+		{
+			(GInterfaceInitFunc) ephy_window_link_iface_init,
+			NULL,
+			NULL
+		};
 
                 type = g_type_register_static (GTK_TYPE_WINDOW,
 					       "EphyWindow",
 					       &our_info, 0);
+
+		g_type_add_interface_static (type,
+					     EPHY_TYPE_LINK,
+					     &link_info);
         }
 
         return type;
 }
 
 static void
-update_fullscreen_popup (EphyWindow *window)
+destroy_fullscreen_popup (EphyWindow *window)
 {
-	GtkWidget *popup = window->priv->exit_fullscreen_popup;
-	int popup_width, popup_height;
-	GdkRectangle screen_rect;
-
-	g_return_if_fail (popup != NULL);
-
-	popup_width = popup->requisition.width;
-	popup_height = popup->requisition.height;
-
-	/* FIXME multihead */
-	gdk_screen_get_monitor_geometry (gdk_screen_get_default (),
-                        gdk_screen_get_monitor_at_window
-                        (gdk_screen_get_default (),
-                         GTK_WIDGET (window)->window),
-                         &screen_rect);
-
-	if (gtk_widget_get_direction (popup) == GTK_TEXT_DIR_RTL)
+	if (window->priv->fullscreen_popup != NULL)
 	{
-		gtk_window_move (GTK_WINDOW (popup),
-				 screen_rect.x + screen_rect.width - popup_width,
-				 screen_rect.height - popup_height);
-	}
-	else
-	{
-		gtk_window_move (GTK_WINDOW (popup),
-                	        screen_rect.x, screen_rect.height - popup_height);
-	}
-}
-
-static void
-screen_size_changed_cb (GdkScreen *screen,
-			EphyWindow *window)
-{
-	update_fullscreen_popup (window);
-}
-
-static void
-destroy_exit_fullscreen_popup (EphyWindow *window)
-{
-	if (window->priv->exit_fullscreen_popup != NULL)
-	{
-		/* FIXME multihead */
-		g_signal_handlers_disconnect_by_func
-			(gdk_screen_get_default (),
-			 G_CALLBACK (screen_size_changed_cb), window);
-
-		gtk_widget_destroy (window->priv->exit_fullscreen_popup);
-		window->priv->exit_fullscreen_popup = NULL;
+		gtk_widget_destroy (window->priv->fullscreen_popup);
+		window->priv->fullscreen_popup = NULL;
 	}
 }
 
@@ -492,7 +469,7 @@ ephy_window_destroy (GtkObject *gtkobject)
 	EphyWindow *window = EPHY_WINDOW (gtkobject);
 	GSList *popups;
 
-	LOG ("EphyWindow destroy %p", window)
+	LOG ("EphyWindow destroy %p", window);
 
 	if (window->priv->closing == FALSE)
 	{
@@ -505,7 +482,7 @@ ephy_window_destroy (GtkObject *gtkobject)
 		ephy_extension_detach_window (manager, window);
 	}
 
-	destroy_exit_fullscreen_popup (window);
+	destroy_fullscreen_popup (window);
 
 	popups = gtk_ui_manager_get_toplevels (window->priv->manager, GTK_UI_MANAGER_POPUP);
 	g_slist_foreach (popups, (GFunc) gtk_menu_shell_deactivate, NULL);
@@ -524,7 +501,7 @@ add_widget (GtkUIManager *manager,
 }
 
 static void
-exit_fullscreen_button_clicked_cb (GtkWidget *button, EphyWindow *window)
+exit_fullscreen_clicked_cb (EphyWindow *window)
 {
 	GtkAction *action;
 
@@ -534,6 +511,13 @@ exit_fullscreen_button_clicked_cb (GtkWidget *button, EphyWindow *window)
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
 }
 
+static gboolean
+get_toolbar_visibility (EphyWindow *window)
+{
+	return ((window->priv->chrome & EPHY_EMBED_CHROME_TOOLBAR) != 0) &&
+	       !window->priv->ppv_mode;
+}			
+
 static void
 get_chromes_visibility (EphyWindow *window, gboolean *show_menubar,
 			gboolean *show_statusbar, gboolean *show_toolbar,
@@ -562,6 +546,7 @@ get_chromes_visibility (EphyWindow *window, gboolean *show_menubar,
 static void
 sync_chromes_visibility (EphyWindow *window)
 {
+	EphyWindowPrivate *priv = window->priv;
 	GtkWidget *menubar;
 	gboolean show_statusbar, show_menubar, show_toolbar, show_bookmarksbar;
 
@@ -572,71 +557,51 @@ sync_chromes_visibility (EphyWindow *window)
 	menubar = gtk_ui_manager_get_widget (window->priv->manager, "/menubar");
 	g_assert (menubar != NULL);
 
-	g_object_set (G_OBJECT (menubar), "visible", show_menubar, NULL);
-	g_object_set (G_OBJECT (window->priv->toolbar), "visible", show_toolbar, NULL);
-	g_object_set (G_OBJECT (window->priv->bookmarksbar), "visible", show_bookmarksbar, NULL);
-	g_object_set (G_OBJECT (window->priv->statusbar), "visible", show_statusbar, NULL);
-}
+	g_object_set (menubar, "visible", show_menubar, NULL);
+	g_object_set (priv->toolbar, "visible", show_toolbar, NULL);
+	g_object_set (priv->bookmarksbar, "visible", show_bookmarksbar, NULL);
+	g_object_set (priv->statusbar, "visible", show_statusbar, NULL);
 
-static void
-fullscreen_popup_size_request_cb (GtkWidget *popup, GtkRequisition *req, EphyWindow *window)
-{
-	update_fullscreen_popup (window);
+	ephy_toolbar_set_lock_visibility (priv->toolbar, !show_statusbar);
+
+	if (priv->fullscreen_popup != NULL)
+	{
+		g_object_set (priv->fullscreen_popup, "visible", !show_toolbar, NULL);
+	}
 }
 
 static void
 ephy_window_fullscreen (EphyWindow *window)
 {
-	GtkWidget *popup, *button, *icon, *label, *hbox;
+	GtkWidget *popup;
+	EphyTab *tab;
+	gboolean lockdown_fs;
 
 	window->priv->fullscreen_mode = TRUE;
 
-	sync_chromes_visibility (window);
+	lockdown_fs = eel_gconf_get_boolean (CONF_LOCKDOWN_FULLSCREEN);
 
-	if (eel_gconf_get_boolean (CONF_LOCKDOWN_FULLSCREEN))
-	{
-		/* no need to show "exit fullscreen" button */
-		return;
-	}
+	popup = ephy_fullscreen_popup_new (window);
+	ephy_fullscreen_popup_set_show_leave
+		(EPHY_FULLSCREEN_POPUP (popup), !lockdown_fs);
+	window->priv->fullscreen_popup = popup;
+	g_signal_connect_swapped (popup, "exit-clicked",
+				  G_CALLBACK (exit_fullscreen_clicked_cb), window);
 
-	popup = gtk_window_new (GTK_WINDOW_POPUP);
-	window->priv->exit_fullscreen_popup = popup;
-
-	button = gtk_button_new ();
-	g_signal_connect (button, "clicked",
-			  G_CALLBACK (exit_fullscreen_button_clicked_cb),
-			  window);
-	gtk_widget_show (button);
-	gtk_container_add (GTK_CONTAINER (popup), button);
-
-	hbox = gtk_hbox_new (FALSE, 2);
-	gtk_widget_show (hbox);
-	gtk_container_add (GTK_CONTAINER (button), hbox);
-
-	icon = gtk_image_new_from_stock (GTK_STOCK_QUIT, GTK_ICON_SIZE_BUTTON);
-	gtk_widget_show (icon);
-	gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, FALSE, 0);
-
-	label = gtk_label_new (_("Exit Fullscreen"));
-	gtk_widget_show (label);
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-
-	gtk_window_set_resizable (GTK_WINDOW (popup), FALSE);
-
-	/* FIXME multihead */
-	g_signal_connect (gdk_screen_get_default (), "size-changed",
-			  G_CALLBACK (screen_size_changed_cb), window);
-	g_signal_connect (popup, "size_request",
-			  G_CALLBACK (fullscreen_popup_size_request_cb), window);
-
-	update_fullscreen_popup (window);
-
-	gtk_widget_show (popup);
+	/* sync status */
+	tab = ephy_window_get_active_tab (window);
+	sync_tab_load_status (tab, NULL, window);
+	sync_tab_security (tab, NULL, window);
 
 	egg_editable_toolbar_set_model
 		(EGG_EDITABLE_TOOLBAR (window->priv->toolbar),
 		 EGG_TOOLBARS_MODEL (
 		 	ephy_shell_get_toolbars_model (ephy_shell, TRUE)));
+
+	ephy_toolbar_set_show_leave_fullscreen (window->priv->toolbar,
+						!lockdown_fs);
+
+	sync_chromes_visibility (window);
 }
 
 static void
@@ -644,7 +609,9 @@ ephy_window_unfullscreen (EphyWindow *window)
 {
 	window->priv->fullscreen_mode = FALSE;
 
-	destroy_exit_fullscreen_popup (window);
+	destroy_fullscreen_popup (window);
+
+	ephy_toolbar_set_show_leave_fullscreen (window->priv->toolbar, FALSE);
 
 	egg_editable_toolbar_set_model
 		(EGG_EDITABLE_TOOLBAR (window->priv->toolbar),
@@ -701,7 +668,7 @@ confirm_close_with_modified_forms (EphyWindow *window)
 		 GTK_DIALOG_MODAL,
 		 GTK_MESSAGE_WARNING,
 		 GTK_BUTTONS_CANCEL,
-		 _("There are unsubmitted changes to form elements."));
+		 _("There are unsubmitted changes to form elements"));
 
 	gtk_message_dialog_format_secondary_text
 		(GTK_MESSAGE_DIALOG (dialog),
@@ -748,10 +715,10 @@ ephy_window_key_press_event (GtkWidget *widget,
 	guint mask = gtk_accelerator_get_default_mod_mask ();
 	char *accel = NULL;
 
-	/* Don't activate menubar in ppv mode */
-	if (window->priv->ppv_mode)
+	/* Don't activate menubar in ppv mode, or in lockdown mode */
+	if (window->priv->ppv_mode || eel_gconf_get_boolean (CONF_LOCKDOWN_HIDE_MENUBAR))
 	{
-		return FALSE;
+		return GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
 	}
 
 	g_object_get (gtk_widget_get_settings (widget),
@@ -1059,7 +1026,7 @@ update_print_actions (EphyWindow *window,
 	action = gtk_action_group_get_action (action_group, "FilePrintSetup");
 	g_object_set (action, "sensitive", printing && print_setup, NULL);
 	action = gtk_action_group_get_action (action_group, "FilePrintPreview");
-	g_object_set (action, "sensitive", enable && printing && print_setup, NULL);
+	g_object_set (action, "sensitive", enable && printing, NULL);
 	action = gtk_action_group_get_action (action_group, "FilePrint");
 	g_object_set (action, "sensitive", enable && printing, NULL);
 
@@ -1105,12 +1072,14 @@ update_actions_sensitivity (EphyWindow *window)
 	g_object_set (action, "sensitive", save_to_disk, NULL);
 	action = gtk_action_group_get_action (popups_action_group, "DownloadLink");
 	g_object_set (action, "sensitive", save_to_disk, NULL);
+	action = gtk_action_group_get_action (popups_action_group, "DownloadLinkAs");
+	g_object_set (action, "sensitive", save_to_disk, NULL);
 	action = gtk_action_group_get_action (popups_action_group, "SaveBackgroundAs");
 	g_object_set (action, "sensitive", save_to_disk, NULL);
 	action = gtk_action_group_get_action (popups_action_group, "SaveImageAs");
 	g_object_set (action, "sensitive", save_to_disk, NULL);
 	action = gtk_action_group_get_action (popups_action_group, "SetImageAsBackground");
-	g_object_set (action, "sensitive", eel_gconf_key_is_writable (CONF_DESKTOP_BG_PICTURE), NULL);
+	g_object_set (action, "sensitive", save_to_disk && eel_gconf_key_is_writable (CONF_DESKTOP_BG_PICTURE), NULL);
 
 	action = gtk_action_group_get_action (action_group, "EditToolbar");
 	g_object_set (action, "sensitive", ! eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_TOOLBAR_EDITING), NULL);
@@ -1208,7 +1177,7 @@ sync_tab_address (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 		address = "";
 	}
 
-	toolbar_set_location (window->priv->toolbar, address);
+	ephy_toolbar_set_location (window->priv->toolbar, address);
 }
 
 static void
@@ -1263,7 +1232,7 @@ sync_tab_icon (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 
 	gtk_window_set_icon (GTK_WINDOW (window), pixbuf);
 
-	toolbar_update_favicon (window->priv->toolbar, address);
+	ephy_toolbar_set_favicon (window->priv->toolbar, address);
 
 	if (pixbuf)
 	{
@@ -1340,8 +1309,8 @@ sync_tab_navigation (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 	action = gtk_action_group_get_action (action_group, "GoForward");
 	g_object_set (action, "sensitive", forward, NULL);
 
-	toolbar_update_navigation_actions (window->priv->toolbar,
-					   back, forward, up);
+	ephy_toolbar_set_navigation_actions (window->priv->toolbar,
+					     back, forward, up);
 }
 
 static void
@@ -1353,6 +1322,7 @@ sync_tab_security (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 	char *state = NULL;
 	char *tooltip;
 	const char *stock_id = STOCK_LOCK_INSECURE;
+	gboolean show_lock = FALSE;
 
 	if (window->priv->closing) return;
 
@@ -1373,6 +1343,7 @@ sync_tab_security (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 		case EPHY_EMBED_STATE_IS_BROKEN:
 			state = _("Broken");
 			stock_id = STOCK_LOCK_BROKEN;
+			show_lock = TRUE;
 			g_free (description);
 			description = NULL;
 			break;
@@ -1387,6 +1358,7 @@ sync_tab_security (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 		case EPHY_EMBED_STATE_IS_SECURE_HIGH:
 			state = _("High");
 			stock_id = STOCK_LOCK_SECURE;
+			show_lock = TRUE;
 			break;
 		default:
 			g_assert_not_reached ();
@@ -1405,8 +1377,18 @@ sync_tab_security (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 
 	ephy_statusbar_set_security_state (EPHY_STATUSBAR (window->priv->statusbar),
 					   stock_id, tooltip);
-	g_free (tooltip);
 
+	ephy_toolbar_set_security_state (window->priv->toolbar,
+					 show_lock, stock_id, tooltip);
+
+	if (window->priv->fullscreen_popup != NULL)
+	{
+		ephy_fullscreen_popup_set_security_state
+			(EPHY_FULLSCREEN_POPUP (window->priv->fullscreen_popup),
+			 show_lock, stock_id, tooltip);
+	}
+
+	g_free (tooltip);
 }
 
 static void
@@ -1483,13 +1465,13 @@ sync_tab_load_status (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 	/* disable print while loading, see bug #116344 */
 	update_print_actions (window, !status);
 
-	if (status)
+	ephy_toolbar_set_spinning (window->priv->toolbar, status);
+
+	if (window->priv->fullscreen_popup)
 	{
-		toolbar_spinner_start (window->priv->toolbar);
-	}
-	else
-	{
-		toolbar_spinner_stop (window->priv->toolbar);
+		ephy_fullscreen_popup_set_spinning
+			 (EPHY_FULLSCREEN_POPUP (window->priv->fullscreen_popup),
+			  status);
 	}
 }
 
@@ -1569,7 +1551,7 @@ sync_tab_zoom (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 		can_zoom_normal = TRUE;
 	}
 
-	toolbar_update_zoom (window->priv->toolbar, can_zoom, zoom);
+	ephy_toolbar_set_zoom (window->priv->toolbar, can_zoom, zoom);
 
 	action_group = window->priv->action_group;
 	action = gtk_action_group_get_action (action_group, "ViewZoomIn");
@@ -1744,7 +1726,7 @@ show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 	const GValue *value;
 	gboolean framed, has_background, can_open_in_new;
 	GtkWidget *widget;
-	EphyEmbedEventType type;
+	guint button;
 
 	/* Do not show the menu in print preview mode */
 	if (window->priv->ppv_mode)
@@ -1760,7 +1742,7 @@ show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 
 	context = ephy_embed_event_get_context (event);
 
-	LOG ("show_embed_popup context %x", context)
+	LOG ("show_embed_popup context %x", context);
 
 	if ((context & EPHY_EMBED_CONTEXT_EMAIL_LINK) &&
 	    (context & EPHY_EMBED_CONTEXT_IMAGE))
@@ -1791,6 +1773,12 @@ show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 		popup = "/EphyInputPopup";
 		update_edit_actions_sensitivity (window, FALSE);
 	}
+	else if (window->priv->fullscreen_mode)
+	{
+		popup = framed ? "/EphyFullscreenFramedDocumentPopup" :
+				 "/EphyFullscreenDocumentPopup";
+		update_edit_actions_sensitivity (window, TRUE);
+	}
 	else
 	{
 		popup = framed ? "/EphyFramedDocumentPopup" :
@@ -1819,8 +1807,8 @@ show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 	g_signal_connect (widget, "hide",
 			  G_CALLBACK (hide_embed_popup_cb), window);
 
-	type = ephy_embed_event_get_event_type (event);
-	if (type == EPHY_EMBED_EVENT_KEY)
+	button = ephy_embed_event_get_button (event);
+	if (button == 0)
 	{
 		gtk_menu_popup (GTK_MENU (widget), NULL, NULL,
 				popup_menu_at_coords, window, 0,
@@ -1830,7 +1818,7 @@ show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 	else
 	{
 		gtk_menu_popup (GTK_MENU (widget), NULL, NULL,
-				NULL, NULL, 2,
+				NULL, NULL, button,
 				gtk_get_current_event_time ());
 	}
 }
@@ -1842,17 +1830,13 @@ tab_context_menu_cb (EphyEmbed *embed,
 {
 	EphyTab *tab;
 
-	g_return_val_if_fail (EPHY_IS_WINDOW (window), FALSE);
-	g_return_val_if_fail (EPHY_IS_EMBED (embed), FALSE);
-	g_return_val_if_fail (EPHY_IS_EMBED_EVENT(event), FALSE);
-
 	tab = ephy_tab_for_embed (embed);
 	g_return_val_if_fail (EPHY_IS_TAB (tab), FALSE);
 	g_return_val_if_fail (window->priv->active_tab == tab, FALSE);
 
 	show_embed_popup (window, tab, event);
 
-	return FALSE;
+	return TRUE;
 }
 
 static void
@@ -1986,7 +1970,7 @@ ephy_window_set_active_tab (EphyWindow *window, EphyTab *new_tab)
 		embed = ephy_tab_get_embed (new_tab);
 		g_signal_connect_object (embed, "ge_context_menu",
 					 G_CALLBACK (tab_context_menu_cb),
-					 window, 0);
+					 window, G_CONNECT_AFTER);
 
 		action = GTK_TOGGLE_ACTION (ephy_tab_get_action (new_tab));
 		gtk_toggle_action_set_active (action, TRUE);
@@ -2045,7 +2029,7 @@ modal_alert_cb (EphyEmbed *embed,
 
 	/* make sure the location entry shows the real URL of the tab's page */
 	address = ephy_embed_get_location (embed, TRUE);
-	toolbar_set_location (window->priv->toolbar, address);
+	ephy_toolbar_set_location (window->priv->toolbar, address);
 	g_free (address);
 
 	/* don't suppress alert */
@@ -2129,7 +2113,7 @@ tab_added_cb (EphyNotebook *notebook,
 
 	update_tabs_menu_sensitivity (window);
 
-	g_signal_connect_object (G_OBJECT (tab), "notify::visible",
+	g_signal_connect_object (G_OBJECT (tab), "notify::visibility",
 				 G_CALLBACK (sync_tab_visibility), window, 0);
 
 	embed = ephy_tab_get_embed (tab);
@@ -2334,31 +2318,41 @@ ephy_window_get_property (GObject *object,
 }
 
 static gboolean
-ephy_window_focus_in_event (GtkWidget *widget, GdkEventFocus *event)
+ephy_window_focus_in_event (GtkWidget *widget,
+			    GdkEventFocus *event)
 {
 	EphyWindow *window = EPHY_WINDOW (widget);
+	EphyWindowPrivate *priv = window->priv;
 
-	if (window->priv->exit_fullscreen_popup)
+	if (priv->fullscreen_popup && !get_toolbar_visibility (window))
 	{
-		gtk_widget_show (window->priv->exit_fullscreen_popup);
+		gtk_widget_show (priv->fullscreen_popup);
 	}
 
 	return GTK_WIDGET_CLASS (parent_class)->focus_in_event (widget, event);
 }
 
 static gboolean
-ephy_window_focus_out_event (GtkWidget *widget, GdkEventFocus *event)
+ephy_window_focus_out_event (GtkWidget *widget,
+			     GdkEventFocus *event)
 {
 	EphyWindow *window = EPHY_WINDOW (widget);
+	EphyWindowPrivate *priv = window->priv;
 
-	if (window->priv->exit_fullscreen_popup)
+	if (priv->fullscreen_popup)
 	{
-		gtk_widget_hide (window->priv->exit_fullscreen_popup);
+		gtk_widget_hide (priv->fullscreen_popup);
 	}
 
 	return GTK_WIDGET_CLASS (parent_class)->focus_out_event (widget, event);
 }
 
+static void
+ephy_window_link_iface_init (EphyLinkIface *iface)
+{
+	iface->open_link = ephy_window_open_link;
+}
+
 static void
 ephy_window_class_init (EphyWindowClass *klass)
 {
@@ -2481,12 +2475,12 @@ action_request_forward_cb (GObject *toolbar,
 }
 
 static EphyTab *
-open_link_cb (EphyLink *link,
-	      const char *address,
-	      EphyTab *tab,
-	      EphyLinkFlags flags,
-	      EphyWindow *window)
+ephy_window_open_link (EphyLink *link,
+		       const char *address,
+		       EphyTab *tab,
+		       EphyLinkFlags flags)
 {
+	EphyWindow *window = EPHY_WINDOW (link);
 	EphyTab *new_tab;
 
 	g_return_val_if_fail (address != NULL, NULL);
@@ -2541,7 +2535,7 @@ ephy_window_init (EphyWindow *window)
 	EggToolbarsModel *model;
 	GError *error = NULL;
 
-	LOG ("EphyWindow initialising %p", window)
+	LOG ("EphyWindow initialising %p", window);
 
 	g_object_ref (ephy_shell);
 
@@ -2555,8 +2549,8 @@ ephy_window_init (EphyWindow *window)
 	setup_ui_manager (window);
 
 	window->priv->notebook = setup_notebook (window);
-	g_signal_connect (window->priv->notebook, "open-link",
-			  G_CALLBACK (open_link_cb), window);
+	g_signal_connect_swapped (window->priv->notebook, "open-link",
+				  G_CALLBACK (ephy_link_open), window);
 	gtk_box_pack_start (GTK_BOX (window->priv->main_vbox),
 			    GTK_WIDGET (window->priv->notebook),
 			    TRUE, TRUE, 0);
@@ -2579,12 +2573,14 @@ ephy_window_init (EphyWindow *window)
 	model= EGG_TOOLBARS_MODEL (ephy_shell_get_toolbars_model (ephy_shell, FALSE));
 
 	/* create the toolbars */
-	window->priv->toolbar = toolbar_new (window);
-	g_signal_connect (window->priv->toolbar, "open-link",
-			  G_CALLBACK (open_link_cb), window);
+	window->priv->toolbar = ephy_toolbar_new (window);
+	g_signal_connect_swapped (window->priv->toolbar, "open-link",
+				  G_CALLBACK (ephy_link_open), window);
+	g_signal_connect_swapped (window->priv->toolbar, "exit-clicked",
+				  G_CALLBACK (exit_fullscreen_clicked_cb), window);
 	window->priv->bookmarksbar = ephy_bookmarksbar_new (window);
-	g_signal_connect (window->priv->bookmarksbar, "open-link",
-			  G_CALLBACK (open_link_cb), window);
+	g_signal_connect_swapped (window->priv->bookmarksbar, "open-link",
+				  G_CALLBACK (ephy_link_open), window);
 
 	g_signal_connect_swapped (window->priv->toolbar, "activation-finished",
 				  G_CALLBACK (sync_chromes_visibility), window);
@@ -2602,12 +2598,12 @@ ephy_window_init (EphyWindow *window)
 	window->priv->tabs_menu = ephy_tabs_menu_new (window);
 	window->priv->enc_menu = ephy_encoding_menu_new (window);
 	window->priv->fav_menu = ephy_favorites_menu_new (window);
-	g_signal_connect (window->priv->fav_menu, "open-link",
-			  G_CALLBACK (open_link_cb), window);
+	g_signal_connect_swapped (window->priv->fav_menu, "open-link",
+				  G_CALLBACK (ephy_link_open), window);
 	window->priv->bmk_menu = ephy_bookmarks_menu_new (window->priv->manager,
 							  BOOKMARKS_MENU_PATH);
-	g_signal_connect (window->priv->bmk_menu, "open-link",
-			  G_CALLBACK (open_link_cb), window);
+	g_signal_connect_swapped (window->priv->bmk_menu, "open-link",
+				  G_CALLBACK (ephy_link_open), window);
 
 	/* forward the toolbar's action_request signal to the bookmarks toolbar,
 	 * so the user can also have bookmarks on the normal toolbar
@@ -2747,7 +2743,7 @@ ephy_window_finalize (GObject *object)
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
 
-	LOG ("Ephy Window finalized %p", window)
+	LOG ("Ephy Window finalized %p", window);
 
 	g_object_unref (ephy_shell);
 }
@@ -3008,15 +3004,9 @@ void
 ephy_window_load_url (EphyWindow *window,
 		      const char *url)
 {
-	EphyEmbed *embed;
+	g_return_if_fail (url != NULL);
 
-        g_return_if_fail (EPHY_IS_WINDOW(window));
-	embed = ephy_window_get_active_embed (window);
-        g_return_if_fail (embed != NULL);
-        g_return_if_fail (url != NULL);
-
-        ephy_embed_load_url (embed, url);
-	ephy_embed_activate (embed);
+	ephy_link_open (EPHY_LINK (window), url, NULL, 0);
 }
 
 /**
@@ -3028,7 +3018,12 @@ ephy_window_load_url (EphyWindow *window,
 void
 ephy_window_activate_location (EphyWindow *window)
 {
-	toolbar_activate_location (window->priv->toolbar);
+	if (window->priv->fullscreen_popup)
+	{
+		gtk_widget_hide (window->priv->fullscreen_popup);
+	}
+
+	ephy_toolbar_activate_location (window->priv->toolbar);
 }
 
 static void
