@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: window-commands.c,v 1.146 2004/12/18 22:16:40 chpe Exp $
+ *  $Id: window-commands.c,v 1.154 2005/01/12 19:04:48 chpe Exp $
  */
 
 #include "config.h"
@@ -38,13 +38,12 @@
 #include "ephy-new-bookmark.h"
 #include "ephy-file-chooser.h"
 #include "ephy-file-helpers.h"
-#include "toolbar.h"
+#include "ephy-toolbar.h"
 #include "ephy-state.h"
 #include "ephy-gui.h"
 #include "ephy-zoom.h"
-#include "egg-toolbars-model.h"
-#include "egg-editable-toolbar.h"
-#include "egg-toolbar-editor.h"
+#include "ephy-notebook.h"
+#include "ephy-toolbar-editor.h"
 
 #include <string.h>
 #include <glib.h>
@@ -59,12 +58,6 @@
 #include <gtk/gtktoggleaction.h>
 #include <glib/gi18n.h>
 
-#define TOOLBAR_EDITOR_KEY	"EphyToolbarEditor"
-enum
-{
-	RESPONSE_ADD_TOOLBAR
-};
-
 void
 window_cmd_edit_find (GtkAction *action,
 		      EphyWindow *window)
@@ -174,7 +167,7 @@ window_cmd_file_send_to	(GtkAction *action,
                            "?Subject=", title,
                            "&Body=", location, NULL);
 
-	ephy_embed_load_url (embed, url);
+	gnome_vfs_url_show (url);
 
 	g_free (title);
 	g_free (location);
@@ -195,26 +188,6 @@ window_cmd_go_forward (GtkAction *action,
 	ephy_embed_go_forward (embed);
 }
 
-void
-window_cmd_go_home (GtkAction *action,
-		    EphyWindow *window)
-{
-	char *location;
-
-	location = eel_gconf_get_string (CONF_GENERAL_HOMEPAGE);
-
-	if (location == NULL || location[0] == '\0')
-	{
-		g_free (location);
-
-		location = g_strdup ("about:blank");
-	}
-
-	ephy_window_load_url (window, location);
-
-	g_free (location);
-}
-
 void
 window_cmd_go_location (GtkAction *action,
 		        EphyWindow *window)
@@ -422,7 +395,7 @@ window_cmd_file_save_as (GtkAction *action,
 	ephy_embed_persist_set_fc_title (persist, _("Save As"));
 	ephy_embed_persist_set_fc_parent (persist,GTK_WINDOW (window));
 	ephy_embed_persist_set_flags
-		(persist, EMBED_PERSIST_MAINDOC | EMBED_PERSIST_ASK_DESTINATION);
+		(persist, EPHY_EMBED_PERSIST_MAINDOC | EPHY_EMBED_PERSIST_ASK_DESTINATION);
 	ephy_embed_persist_set_persist_key
 		(persist, CONF_STATE_SAVE_DIR);
 
@@ -711,8 +684,8 @@ save_temp_source (EphyEmbed *embed)
 		(ephy_embed_factory_new_object (EPHY_TYPE_EMBED_PERSIST));
 
 	ephy_embed_persist_set_embed (persist, embed);
-	ephy_embed_persist_set_flags (persist, EMBED_PERSIST_COPY_PAGE |
-				      EMBED_PERSIST_NO_VIEW);
+	ephy_embed_persist_set_flags (persist, EPHY_EMBED_PERSIST_COPY_PAGE |
+				      EPHY_EMBED_PERSIST_NO_VIEW);
 	ephy_embed_persist_set_dest (persist, tmp);
 
 	g_signal_connect (persist, "completed",
@@ -787,95 +760,11 @@ window_cmd_edit_prefs (GtkAction *action,
 	ephy_dialog_show (dialog);
 }
 
-static void
-toolbar_editor_destroy_cb (GtkWidget *tbe,
-			   EphyWindow *window)
-{
-	egg_editable_toolbar_set_edit_mode (EGG_EDITABLE_TOOLBAR
-		(ephy_window_get_toolbar (window)), FALSE);
-	egg_editable_toolbar_set_edit_mode (EGG_EDITABLE_TOOLBAR
-		(ephy_window_get_bookmarksbar (window)), FALSE);
-
-	g_object_set_data (G_OBJECT (window), TOOLBAR_EDITOR_KEY, NULL);
-}
-
-static void
-toolbar_editor_response_cb (GtkDialog  *dialog,
-			    gint response_id,
-			    EggToolbarsModel *model)
-{
-	switch (response_id)
-	{
-	case GTK_RESPONSE_CLOSE:
-		gtk_widget_destroy (GTK_WIDGET (dialog));
-		break;
-	case RESPONSE_ADD_TOOLBAR:
-		egg_toolbars_model_add_toolbar (model, -1, "UserCreated");
-		break;
-	case GTK_RESPONSE_HELP:
-		ephy_gui_help (GTK_WINDOW (dialog), "epiphany", "to-edit-toolbars");
-		break;
-	}
-}
-
 void
 window_cmd_edit_toolbar (GtkAction *action,
 			 EphyWindow *window)
 {
-	GtkWidget *editor, *dialog, *toolbar;
-	EggToolbarsModel *model;
-
-	dialog = GTK_WIDGET (g_object_get_data (G_OBJECT (window), TOOLBAR_EDITOR_KEY));
-	if (dialog != NULL)
-	{
-		gtk_window_present (GTK_WINDOW (dialog));
-		return;
-	}
-
-	model = EGG_TOOLBARS_MODEL
-		(ephy_shell_get_toolbars_model (ephy_shell, FALSE));
-	toolbar = ephy_window_get_toolbar (window);
-
-	dialog = gtk_dialog_new ();
-	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-	gtk_window_set_title (GTK_WINDOW (dialog), _("Toolbar Editor"));
-        gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
-	gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
-	gtk_window_set_icon_name (GTK_WINDOW (dialog), "web-browser");
-
-	editor = egg_toolbar_editor_new (GTK_UI_MANAGER (ephy_window_get_ui_manager (window)), model);
-	egg_toolbar_editor_load_actions (EGG_TOOLBAR_EDITOR (editor),
-					 ephy_file ("epiphany-toolbar.xml"));
-	gtk_container_set_border_width (GTK_CONTAINER (EGG_TOOLBAR_EDITOR (editor)), 5);
-	gtk_box_set_spacing (GTK_BOX (EGG_TOOLBAR_EDITOR (editor)), 5);
-	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), editor);
-	g_signal_connect (editor, "destroy",
-			  G_CALLBACK (toolbar_editor_destroy_cb), window);
-	gtk_widget_show (editor);
-	
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
-
-	gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)), 5);
-	
-	gtk_dialog_add_button (GTK_DIALOG (dialog),
-			       _("_Add a New Toolbar"), RESPONSE_ADD_TOOLBAR);
-	gtk_dialog_add_button (GTK_DIALOG (dialog),
-			       GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
-	gtk_dialog_add_button (GTK_DIALOG (dialog),
-			       GTK_STOCK_HELP, GTK_RESPONSE_HELP);
-	
-	g_signal_connect (G_OBJECT (dialog), "response",
-			  G_CALLBACK (toolbar_editor_response_cb), model);
-	ephy_state_add_window (dialog, "toolbar_editor",
-		               500, 330, FALSE,
-			       EPHY_STATE_WINDOW_SAVE_SIZE);
-	gtk_widget_show (dialog);
-
-	egg_editable_toolbar_set_edit_mode (EGG_EDITABLE_TOOLBAR (toolbar), TRUE);
-	egg_editable_toolbar_set_edit_mode
-		(EGG_EDITABLE_TOOLBAR (ephy_window_get_bookmarksbar (window)), TRUE);
-
-	g_object_set_data (G_OBJECT (window), TOOLBAR_EDITOR_KEY, dialog);
+	ephy_toolbar_editor_show (window);
 }
 
 void
@@ -907,9 +796,17 @@ window_cmd_help_about (GtkAction *action,
 	gtk_show_about_dialog (GTK_WINDOW (window),
 			       "name", _("Epiphany"),
 			       "version", VERSION,
-			       "copyright", "Copyright \xc2\xa9 2002-2004 Marco Pesenti Gritti",
+			       "copyright", "Copyright \xc2\xa9 2002-2005 Marco Pesenti Gritti",
 			       "authors", authors,
 			       "documenters", documenters,
+				/* Translators: This is a special message that shouldn't be translated
+				 * literally. It is used in the about box to give credits to
+				 * the translators.
+				 * Thus, you should translate it to your name and email address.
+				 * You should also include other translators who have contributed to
+				 * this translation; in that case, please write each of them on a separate
+				 * line seperated by newlines (\n).
+				 */
 			       "translator-credits", _("translator-credits"),
 			       "logo-icon-name", "web-browser",
 			       "website", "http://www.gnome.org/projects/epiphany",
@@ -1019,11 +916,11 @@ void
 window_cmd_load_location (GtkAction *action,
 			  EphyWindow *window)
 {
-	Toolbar *toolbar;
+	EphyToolbar *toolbar;
 	const char *location;
 
 	toolbar = EPHY_TOOLBAR (ephy_window_get_toolbar (window));
-	location = toolbar_get_location (toolbar);
+	location = ephy_toolbar_get_location (toolbar);
 
 	if (location)
 	{
